@@ -1,6 +1,7 @@
 /* slideshow.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -177,6 +178,10 @@ void slideshow_change_image(winwidget winwid, int change)
 	gib_list *last = NULL;
 	int i = 0;
 	int jmp = 1;
+	/* We can't use filelist_len in the for loop, since that changes when we
+	 * encounter invalid images.
+	 */
+	int our_filelist_len = filelist_len;
 	char *s;
 
 	/* Without this, clicking a one-image slideshow reloads it. Not very *
@@ -197,7 +202,7 @@ void slideshow_change_image(winwidget winwid, int change)
 	}
 
 	/* The for loop prevents us looping infinitely */
-	for (i = 0; i < filelist_len; i++) {
+	for (i = 0; i < our_filelist_len; i++) {
 		winwidget_free_image(winwid);
 		switch (change) {
 		case SLIDE_NEXT:
@@ -311,7 +316,7 @@ void feh_action_run(feh_file * file, char *action)
 {
 	if (action) {
 		char *sys;
-		D(3, ("Running action %s\n", action));
+		D(("Running action %s\n", action));
 		sys = feh_printf(action, file);
 
 		if (opt.verbose && !opt.list && !opt.customlist)
